@@ -82,9 +82,9 @@
 #define   MCHelpSwIDP        L"\n  id[c,d,p,q]   Disable messages"
 #define   MCHelpSwIEML       L"\n  ieml[addr]    Send archive by email"
 #define   MCHelpSwIERR       L"\n  ierr          Send all messages to stderr"
-#define   MCHelpSwILOG       L"\n  ilog[name]    Log errors to file (registered versions only)"
+#define   MCHelpSwILOG       L"\n  ilog[name]    Log errors to file"
 #define   MCHelpSwINUL       L"\n  inul          Disable all messages"
-#define   MCHelpSwIOFF       L"\n  ioff          Turn PC off after completing an operation"
+#define   MCHelpSwIOFF       L"\n  ioff[n]       Turn PC off after completing an operation"
 #define   MCHelpSwISND       L"\n  isnd          Enable sound"
 #define   MCHelpSwIVER       L"\n  iver          Display the version number"
 #define   MCHelpSwK          L"\n  k             Lock archive"
@@ -294,7 +294,8 @@
 #define   MRprNoFiles        L"\nNo files found"
 #define   MLogUnexpEOF       L"\nUnexpected end of archive"
 #define   MRepAskReconst     L"\nReconstruct archive structure ?"
-#define   MRecScanning       L"\nScanning..."
+#define   MRRSearch          L"\nSearching for recovery record"
+#define   MAnalyzeFileData   L"\nAnalyzing file data"
 #define   MRecRNotFound      L"\nData recovery record not found"
 #define   MRecRFound         L"\nData recovery record found"
 #define   MRecSecDamage      L"\nSector %ld (offsets %lX...%lX) damaged"
@@ -310,8 +311,6 @@
 #define   MErrLnkRead        L"\nWARNING: Cannot read symbolic link %s"
 #define   MSymLinkExists     L"\nWARNING: Symbolic link %s already exists"
 #define   MAskRetryCreate    L"\nCannot create %s. Retry ?"
-#define   MListMACHead1      L"\n Mac OS file type:  %c%c%c%c  ; "
-#define   MListMACHead2      L"file creator:  %c%c%c%c\n"
 #define   MDataBadCRC        L"\n%-20s : packed data checksum error in volume %s"
 #define   MFileRO            L"\n%s is read-only"
 #define   MACLGetError       L"\nWARNING: Cannot get %s security data\n"
@@ -343,7 +342,7 @@
 #define   MFAT32Size         L"\nWARNING: FAT32 file system does not support 4 GB or larger files"
 #define   MErrChangeAttr     L"\nWARNING: Cannot change attributes of %s"
 #define   MWrongSFXVer       L"\nERROR: default SFX module does not support RAR %d.%d archives"
-#define   MCannotEncName     L"\nCannot encrypt archive already contained encrypted files"
+#define   MCannotEncName     L"\nCannot encrypt archive already containing encrypted files"
 #define   MCannotEmail       L"\nCannot email the file %s"
 #define   MCopyrightS        L"\nRAR SFX archive" 
 #define   MSHelpCmd          L"\n\n<Commands>" 
@@ -353,6 +352,7 @@
 #define   MRecVolLimit       L"\nTotal number of usual and recovery volumes must not exceed %d"
 #define   MVolumeNumber      L"volume %d"
 #define   MCannotDelete      L"\nCannot delete %s"
+#define   MRecycleFailed     L"\nCannot move some files and folders to Recycle Bin"
 #define   MCalcCRC           L"\nCalculating the checksum"
 #define   MTooLargeSFXArc    L"\nWARNING: Too large SFX archive. Windows cannot run the executable file exceeding 4 GB."
 #define   MCalcCRCAllVol     L"\nCalculating checksums of all volumes."
@@ -360,9 +360,8 @@
 #define   MNewerRAR          L"\nYou may need a newer version of RAR."
 #define   MUnkEncMethod      L"\nUnknown encryption method in %s"
 #define   MWrongPassword     L"\nThe specified password is incorrect."
-#define   MRepairing         L"\nRepairing"
 #define   MAreaDamaged       L"\nCorrupt %d bytes at %08x %08x"
-#define   MBlocksRecovered   L"\n%d blocks recovered."
+#define   MBlocksRecovered   L"\n%u blocks are recovered, %u blocks are relocated"
 #define   MRRDamaged         L"\nRecovery record is corrupt."
 #define   MTestingRR         L"\nTesting the recovery record"
 #define   MFailed            L"Failed"
